/*
 * @(#)XGridHello.java created Mar 29, 2006 Casalino
 *
 * Copyright (c) 1996-2006 Luca Lutterotti All Rights Reserved.
 *
 * This software is the research result of Luca Lutterotti and it is 
 * provided as it is as confidential and proprietary information.  
 * You shall not disclose such Confidential Information and shall use 
 * it only in accordance with the terms of the license agreement you 
 * entered into with the author.
 *
 * THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

package it.unitn.ing.xgridclient;

import base64.Base64;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.*;

/**
 * The XGridHello is a class to show how to use the XGridClient.
 * Compile it and put it in a jar with the name: xgridhello.jar
 *
 *
 * @author Luca Lutterotti
 * @version $Revision: 1.0 $, $Date: Mar 29, 2006 9:59:42 AM $
 * @since JDK1.1
 */

public class XGridHello {

  public static String[] xgridFilenames = {"xgridhello.jar", "Hello.txt"};
  public static String baseJarBase64 = null;
  public static String[] javaArguments = null;
  public static String javaCommand = "/usr/bin/java";
  public static int runningTasks = 0;

  public static boolean retrieveData = true;

  /**
  * This is an example of how to use the XGridClient to submit java jobs
  *
  **/

  public static void main(String[] args) {
    if (args != null && args.length > 0 && args[0].equalsIgnoreCase("-hello")) {

      // this is the work done when invoked from the the Agent

      try {
        String hostName = InetAddress.getLocalHost().getCanonicalHostName();
        System.out.println("Hello world from " + hostName);
      } catch (UnknownHostException e) {
        System.out.println("Hello world from unknown!");
      }
      if ((new File(xgridFilenames[1])).exists()) {
        System.out.println("The file " + xgridFilenames[1] + " is correctly present here.");
      }

    } else {

      // the XGrid main job submission

      System.out.println("Initializing and looking for the controller......");
      XGridHello xgridHello = new XGridHello();
      xgridHello.initXGrid();

      // submitting ten times the same job only a different name

      retrieveData = false; // we do not want to retrieve data immediately (as an
                            // example, you may prefer retrieving it immediately
      for (int i = 0; i < 10; i++) {
        xgridHello.submitJob(i);
      }

      System.out.println("Tasks submitted, we will wait 10 secs before starting asking for the results.");

      try {
        Thread.sleep(10000); // we wait 10 secs before retrieving the data
      } catch (InterruptedException ie) {
        ie.printStackTrace(System.err);
      }
      retrieveData = true; // now we retrieve everything

      while (runningTasks > 0) { // now we don't exit from the program until all tasks
                               // are done
        try {
          Thread.sleep(500);
        } catch (InterruptedException ie) {
          ie.printStackTrace(System.err);
        }
      }

    }
    System.exit(0);
  }

  static void initXGrid() {
    // initialize the XGrid and check for available controllers around
    XGridClient.checkController();

    // now we ask for the list of controllers found to print it
    // we shows also how to choose for example the last controller in the list
    String[] controllerHostList = XGridClient.getControllersList();
    if (controllerHostList == null || controllerHostList.length < 1) {
      System.out.println("No XGrid controllers found! Exiting....");
      System.exit(0);
    }
    System.out.println("List of available controllers, we choose the last one");
    for (int i = 0; i < controllerHostList.length; i++)
      System.out.println(controllerHostList[i]);
    // we select the last available controller
    XGridClient.setControllerHost(controllerHostList[controllerHostList.length - 1]);

    // we pre-encode the jar file and setup file names and java arguments
    if (baseJarBase64 == null) {
      // this load the jar file and return it as base64 encoded
      baseJarBase64 = XGridClientMessage.loadfile(xgridFilenames[0]);
      javaArguments = new String[]{"-mx128M", "-jar", xgridFilenames[0], "-hello", xgridFilenames[1]};
    }
  }


  public void submitJob(final int index) {

    (new Thread() {
      public void run() {
        String resultData = null;

        String[] filesBase64ToSubmit = new String[2];
        filesBase64ToSubmit[0] = baseJarBase64;
        // we do not need the real file, we just use the file content and the file Hello.txt will
        // be created on the agent only. If you have a real file, then you need to read it and
        // encode base64 as we do here for the String
        filesBase64ToSubmit[1] = getStringAsBase64String("This is the content of the file, Hello.txt");
        boolean success = false;
        runningTasks++;
        while (!success) {
          System.out.println("Submitting job number " + index);
          String clientName = "XGridClient_" + index;
          String jobId = XGridClient.submitJob(clientName,
              "myJob_number_" + index,
              xgridFilenames, filesBase64ToSubmit, javaCommand,
              javaArguments);
          if (!jobId.equals(Client.FAILED)) {
            // if retrieveData is false, we do not start retrieving results until it becomes true
            // you can use it to decide if the data should be retrieved soon or later
            while (!retrieveData) {
              try {
                Thread.sleep(1000);
              } catch (InterruptedException ie) {
                ie.printStackTrace(System.err);
              }
            }
            System.out.println("Retrieving data for job number " + index);
            // here we retrieve the results
            resultData = XGridClient.getResults(clientName, jobId);
            // we print the results on the console
            System.out.println(resultData);
            if (!resultData.equals(Client.CANCELED) && !resultData.equals(Client.FAILED)) {
              success = true; // we received the correct data, the job has been successful
            } else {
                // we got a problem here, the job was cancelled or failed, we wait 10 secs and resubmit it
                try {
                  Thread.sleep(10000);
                } catch (InterruptedException ie) {
                  ie.printStackTrace(System.err);
                }
            }
          } else {
            // we got a problem here, the job failed, we wait 10 secs and resubmit it
            try {
              Thread.sleep(10000);
            } catch (InterruptedException ie) {
              ie.printStackTrace(System.err);
            }
          }
        }
        runningTasks--;
      }
    }).start();
    // we pause a little before submitting the next job
    try {
      Thread.sleep(500);
    } catch (InterruptedException ie) {
      ie.printStackTrace(System.err);
    }
  }

  public String getStringAsBase64String(String aTextString) {
    ByteArrayOutputStream stream = new ByteArrayOutputStream();
    BufferedWriter br = new BufferedWriter(new PrintWriter(stream));
    try {
      br.write(aTextString);
    } catch (IOException ie) {
      ie.printStackTrace(System.err);
    }
    return Base64.encodeBytes(stream.toByteArray());
  }

}
